from typing import Optional

from cpl_core.database.context import DatabaseContextABC
from cpl_query.extension import List

from bot_core.logging.database_logger import DatabaseLogger
from bot_data.abc.game_server_repository_abc import GameServerRepositoryABC
from bot_data.abc.user_game_ident_repository_abc import (
    UserGameIdentRepositoryABC,
)
from bot_data.abc.user_repository_abc import UserRepositoryABC
from bot_data.model.user_game_ident import UserGameIdent


class UserGameIdentRepositoryService(UserGameIdentRepositoryABC):
    def __init__(
        self,
        logger: DatabaseLogger,
        db_context: DatabaseContextABC,
        users: UserRepositoryABC,
        game_servers: GameServerRepositoryABC,
    ):
        self._logger = logger
        self._context = db_context

        self._users = users
        self._game_servers = game_servers

        UserGameIdentRepositoryABC.__init__(self)

    def _from_result(self, result: tuple) -> UserGameIdent:
        return UserGameIdent(
            self._users.get_user_by_id(result[1]),
            self._game_servers.get_game_server_by_id(result[2]),
            result[3],
            result[4],
            result[5],
            id=result[0],
        )

    def get_user_game_idents(self) -> List[UserGameIdent]:
        joins = List(UserGameIdent)
        self._logger.trace(
            __name__,
            f"Send SQL command: {UserGameIdent.get_select_all_string()}",
        )
        results = self._context.select(UserGameIdent.get_select_all_string())
        for result in results:
            self._logger.trace(__name__, f"Get UserGameIdent with id {result[0]}")
            joins.append(self._from_result(result))

        return joins

    def get_user_game_ident_by_id(self, id: int) -> UserGameIdent:
        self._logger.trace(
            __name__,
            f"Send SQL command: {UserGameIdent.get_select_by_id_string(id)}",
        )
        result = self._context.select(UserGameIdent.get_select_by_id_string(id))[0]
        return self._from_result(result)

    def get_user_game_ident_by_ident(self, ident: str) -> UserGameIdent:
        self._logger.trace(
            __name__,
            f"Send SQL command: {UserGameIdent.get_select_by_ident_string(ident)}",
        )
        result = self._context.select(UserGameIdent.get_select_by_ident_string(ident))[0]
        return self._from_result(result)

    def find_user_game_ident_by_ident(self, ident: str) -> Optional[UserGameIdent]:
        self._logger.trace(
            __name__,
            f"Send SQL command: {UserGameIdent.get_select_by_ident_string(ident)}",
        )
        result = self._context.select(UserGameIdent.get_select_by_ident_string(ident))
        if len(result) == 0:
            return None

        result = result[0]
        return self._from_result(result)

    def get_user_game_idents_by_user_id(self, user_id: int) -> List[UserGameIdent]:
        joins = List(UserGameIdent)
        self._logger.trace(
            __name__,
            f"Send SQL command: {UserGameIdent.get_select_by_user_id_string(user_id)}",
        )
        results = self._context.select(UserGameIdent.get_select_by_user_id_string(user_id))
        for result in results:
            joins.append(self._from_result(result))

        return joins

    def add_user_game_ident(self, user_game_ident: UserGameIdent):
        self._logger.trace(__name__, f"Send SQL command: {user_game_ident.insert_string}")
        self._context.cursor.execute(user_game_ident.insert_string)

    def update_user_game_ident(self, user_game_ident: UserGameIdent):
        self._logger.trace(__name__, f"Send SQL command: {user_game_ident.udpate_string}")
        self._context.cursor.execute(user_game_ident.udpate_string)

    def delete_user_game_ident(self, user_game_ident: UserGameIdent):
        self._logger.trace(__name__, f"Send SQL command: {user_game_ident.delete_string}")
        self._context.cursor.execute(user_game_ident.delete_string)

    def delete_user_game_ident_by_user_id(self, user_id: int):
        self._logger.trace(
            __name__,
            f"Send SQL command: {UserGameIdent.delete_by_user_id_string}",
        )
        self._context.cursor.execute(UserGameIdent.delete_by_user_id_string(user_id))
