from typing import Optional

from cpl_core.database.context import DatabaseContextABC
from cpl_query.extension import List

from bot_core.logging.database_logger import DatabaseLogger
from bot_data.abc.server_repository_abc import ServerRepositoryABC
from bot_data.abc.user_repository_abc import UserRepositoryABC
from bot_data.model.user import User


class UserRepositoryService(UserRepositoryABC):
    def __init__(
        self,
        logger: DatabaseLogger,
        db_context: DatabaseContextABC,
        servers: ServerRepositoryABC,
    ):
        self._logger = logger
        self._context = db_context

        self._servers = servers

        UserRepositoryABC.__init__(self)

    def _from_result(self, result: tuple):
        return User(
            result[1],
            result[2],
            result[3],
            result[4],
            self._servers.get_server_by_id(result[5]),
            result[6],
            result[7],
            id=result[0],
        )

    def get_users(self) -> List[User]:
        users = List(User)
        self._logger.trace(__name__, f"Send SQL command: {User.get_select_all_string()}")
        results = self._context.select(User.get_select_all_string())
        for result in results:
            self._logger.trace(__name__, f"Get user with id {result[0]}")
            users.append(self._from_result(result))

        return users

    def get_user_by_id(self, id: int) -> User:
        self._logger.trace(__name__, f"Send SQL command: {User.get_select_by_id_string(id)}")
        result = self._context.select(User.get_select_by_id_string(id))[0]

        return self._from_result(result)

    def find_user_by_id(self, id: int) -> Optional[User]:
        self._logger.trace(__name__, f"Send SQL command: {User.get_select_by_id_string(id)}")
        result = self._context.select(User.get_select_by_id_string(id))
        if result is None or len(result) == 0:
            return None

        result = result[0]

        return self._from_result(result)

    def get_users_by_discord_id(self, discord_id: int) -> List[User]:
        users = List(User)
        self._logger.trace(
            __name__,
            f"Send SQL command: {User.get_select_by_discord_id_string(discord_id)}",
        )
        results = self._context.select(User.get_select_by_discord_id_string(discord_id))
        for result in results:
            users.append(self._from_result(result))

        return users

    def get_users_by_server_id(self, server_id: int) -> List[User]:
        users = List(User)
        self._logger.trace(
            __name__,
            f"Send SQL command: {User.get_select_by_server_id_string(server_id)}",
        )
        results = self._context.select(User.get_select_by_server_id_string(server_id))
        for result in results:
            users.append(self._from_result(result))

        return users

    def get_user_by_discord_id_and_server_id(self, discord_id: int, server_id: int) -> User:
        self._logger.trace(
            __name__,
            f"Send SQL command: {User.get_select_by_discord_id_and_server_id_string(discord_id, server_id)}",
        )
        result = self._context.select(User.get_select_by_discord_id_and_server_id_string(discord_id, server_id))[0]

        return self._from_result(result)

    def find_user_by_discord_id_and_server_id(self, discord_id: int, server_id: int) -> Optional[User]:
        self._logger.trace(
            __name__,
            f"Send SQL command: {User.get_select_by_discord_id_and_server_id_string(discord_id, server_id)}",
        )
        result = self._context.select(User.get_select_by_discord_id_and_server_id_string(discord_id, server_id))
        if result is None or len(result) == 0:
            return None

        result = result[0]

        return self._from_result(result)

    def add_user(self, user: User):
        self._logger.trace(__name__, f"Send SQL command: {user.insert_string}")
        self._context.cursor.execute(user.insert_string)

    def update_user(self, user: User):
        self._logger.trace(__name__, f"Send SQL command: {user.udpate_string}")
        self._context.cursor.execute(user.udpate_string)

    def delete_user(self, user: User):
        self._logger.trace(__name__, f"Send SQL command: {user.delete_string}")
        self._context.cursor.execute(user.delete_string)
