from cpl_discord.service import DiscordBotServiceABC
from cpl_query.extension import List

from bot_data.model.auto_role import AutoRole
from bot_graphql.abc.filter_abc import FilterABC


class AutoRoleFilter(FilterABC):
    def __init__(
        self,
        bot: DiscordBotServiceABC,
    ):
        FilterABC.__init__(self)

        self._bot = bot

        self._id = None
        self._channel_id = None
        self._channel_name = None
        self._message_id = None
        self._server = None

    def from_dict(self, values: dict):
        if "id" in values:
            self._id = int(values["id"])

        if "channelId" in values:
            self._channel_id = int(values["channelId"])

        if "channelName" in values:
            self._channel_name = values["channelName"]

        if "messageId" in values:
            self._message_id = int(values["messageId"])

        if "server" in values:
            from bot_graphql.filter.server_filter import ServerFilter

            server = ServerFilter()
            server.from_dict(values["server"])
            self._server = server

    def filter(self, query: List[AutoRole]) -> List[AutoRole]:
        if self._id is not None:
            query = query.where(lambda x: x.id == self._id)

        if self._channel_id is not None:
            query = query.where(
                lambda x: x.discord_channel_id == self._channel_id or str(self._channel_id) in str(x.discord_channel_id)
            )

        if self._channel_name is not None:
            query = query.where(
                lambda x: x.discord_channel_name == self._channel_name or self._channel_name in x.discord_channel_name
            )

        if self._message_id is not None:
            query = query.where(
                lambda x: x.discord_message_id == self._message_id or str(self._message_id) in str(x.discord_message_id)
            )

        if self._server is not None:
            servers = self._server.filter(query.select(lambda x: x.server)).select(lambda x: x.id)
            query = query.where(lambda x: x.server.id in servers)

        return query
