from cpl_core.database.context import DatabaseContextABC
from cpl_discord.service import DiscordBotServiceABC

from bot_data.abc.level_repository_abc import LevelRepositoryABC
from bot_data.abc.server_repository_abc import ServerRepositoryABC
from bot_data.abc.user_repository_abc import UserRepositoryABC
from bot_data.model.user_role_enum import UserRoleEnum
from bot_graphql.abc.query_abc import QueryABC
from modules.level.service.level_service import LevelService
from modules.permission.service.permission_service import PermissionService


class UserMutation(QueryABC):
    def __init__(
        self,
        servers: ServerRepositoryABC,
        users: UserRepositoryABC,
        bot: DiscordBotServiceABC,
        db: DatabaseContextABC,
        permissions: PermissionService,
        levels: LevelRepositoryABC,
        level_service: LevelService,
    ):
        QueryABC.__init__(self, "UserMutation")

        self._servers = servers
        self._users = users
        self._bot = bot
        self._db = db
        self._permissions = permissions
        self._levels = levels
        self._level_service = level_service

        self.set_field("updateUser", self.resolve_update_user)

    def resolve_update_user(self, *_, input: dict):
        user = self._users.get_user_by_id(input["id"])
        self._can_user_mutate_data(user.server, UserRoleEnum.moderator)

        new_xp = None
        if "levelId" in input:
            level = self._levels.get_level_by_id(input["levelId"])
            if user.level.id != level.id:
                new_xp = level.min_xp

        user.xp = new_xp if new_xp is not None else input["xp"] if "xp" in input else user.xp

        self._users.update_user(user)
        self._db.save_changes()
        self._bot.loop.create_task(self._level_service.set_level(user))

        user = self._users.get_user_by_id(input["id"])
        return user
