import traceback

from cpl_core.configuration.configuration_model_abc import ConfigurationModelABC
from cpl_core.console import Console
from cpl_query.extension import List


class BaseServerSettings(ConfigurationModelABC):
    def __init__(self):
        ConfigurationModelABC.__init__(self)

        self._id: int = 0
        self._max_voice_state_hours: int = 0
        self._xp_per_message: int = 0
        self._xp_per_reaction: int = 0
        self._max_message_xp_per_hour: int = 0
        self._xp_per_ontime_hour: int = 0
        self._xp_per_event_participation: int = 0
        self._xp_per_achievement: int = 0
        self._afk_channel_ids: List[int] = List(int)
        self._afk_command_channel_id: int = 0
        self._help_command_reference_url: str = ""
        self._help_voice_channel_id: int = 0
        self._team_channel_id: int = 0
        self._ping_urls = List(str)

    @property
    def id(self) -> int:
        return self._id

    @property
    def max_voice_state_hours(self) -> int:
        return self._max_voice_state_hours

    @property
    def xp_per_message(self) -> int:
        return self._xp_per_message

    @property
    def xp_per_reaction(self) -> int:
        return self._xp_per_reaction

    @property
    def max_message_xp_per_hour(self) -> int:
        return self._max_message_xp_per_hour

    @property
    def xp_per_ontime_hour(self) -> int:
        return self._xp_per_ontime_hour

    @property
    def xp_per_event_participation(self) -> int:
        return self._xp_per_event_participation

    @property
    def xp_per_achievement(self) -> int:
        return self._xp_per_achievement

    @property
    def afk_channel_ids(self) -> List[int]:
        return self._afk_channel_ids

    @property
    def afk_command_channel_id(self) -> int:
        return self._afk_command_channel_id

    @property
    def help_command_reference_url(self) -> str:
        return self._help_command_reference_url

    @property
    def team_channel_id(self) -> int:
        return self._team_channel_id

    @property
    def help_voice_channel_id(self) -> int:
        return self._help_voice_channel_id

    @property
    def ping_urls(self) -> List[str]:
        return self._ping_urls

    def from_dict(self, settings: dict):
        try:
            self._id = int(settings["Id"])
            self._max_voice_state_hours = int(settings["MaxVoiceStateHours"])
            self._xp_per_message = int(settings["XpPerMessage"])
            self._xp_per_reaction = int(settings["XpPerReaction"])
            self._max_message_xp_per_hour = int(settings["MaxMessageXpPerHour"])
            self._xp_per_ontime_hour = int(settings["XpPerOntimeHour"])
            self._xp_per_event_participation = (
                0 if "XpPerEventParticipation" not in settings else settings["XpPerEventParticipation"]
            )
            for index in settings["AFKChannelIds"]:
                self._afk_channel_ids.append(int(index))
            self._afk_command_channel_id = settings["AFKCommandChannelId"]
            self._help_command_reference_url = settings["HelpCommandReferenceUrl"]
            self._help_voice_channel_id = settings["HelpVoiceChannelId"]
            self._team_channel_id = settings["TeamChannelId"]
            for url in settings["PingURLs"]:
                self._ping_urls.append(url)
        except Exception as e:
            Console.error(f"[ ERROR ] [ {__name__} ]: Reading error in {type(self).__name__} settings")
            Console.error(f"[ EXCEPTION ] [ {__name__} ]: {e} -> {traceback.format_exc()}")
