from cpl_core.logging import LoggerABC
from cpl_discord.events.on_raw_reaction_add_abc import OnRawReactionAddABC
from cpl_discord.service import DiscordBotServiceABC
from discord import RawReactionActionEvent

from bot_core.helper.event_checks import EventChecks
from modules.level.service.level_service import LevelService


class LevelOnRawReactionAddEvent(OnRawReactionAddABC):
    def __init__(
        self,
        logger: LoggerABC,
        bot: DiscordBotServiceABC,
        level: LevelService,
    ):
        OnRawReactionAddABC.__init__(self)

        self._logger = logger
        self._bot = bot
        self._level = level

    @EventChecks.check_is_ready()
    async def on_raw_reaction_add(self, payload: RawReactionActionEvent):
        self._logger.debug(__name__, f"Module {type(self)} started")
        try:
            self._logger.trace(__name__, f"Handle reaction {payload} for level")

            guild = self._bot.get_guild(payload.guild_id)
            member = guild.get_member(payload.user_id)

            await self._level.check_level(member)
        except Exception as e:
            self._logger.error(__name__, f"Level check by message failed", e)
